#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2020 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Valerio Cosentino <valcos@bitergia.com>
#     inishchith <inishchith@gmail.com>
#

import logging
import os

from graal.graal import (Graal,
                         GraalCommand,
                         GraalError,
                         GraalRepository,
                         DEFAULT_WORKTREE_PATH)
from graal.backends.core.analyzers.analyzer import Analyzer
from graal.backends.core.analyzers.pylint import PyLint
from graal.backends.core.analyzers.flake8 import Flake8
from graal.backends.core.analyzers.jadolint import Jadolint, SMELLS
from perceval.utils import DEFAULT_DATETIME, DEFAULT_LAST_DATETIME

PYLINT = "pylint"
FLAKE8 = "flake8"
JADOLINT = "jadolint"

CATEGORY_COQUA_PYLINT = 'code_quality_' + PYLINT
CATEGORY_COQUA_FLAKE8 = 'code_quality_' + FLAKE8
CATEGORY_COQUA_JADOLINT = 'code_quality_' + JADOLINT

logger = logging.getLogger(__name__)


class CoQua(Graal):
    """CoQua backend.

    This class extends the Graal backend. It gathers
    insights about code quality in Python code.

    :param uri: URI of the Git repository
    :param gitpath: path to the repository or to the log file
    :param worktreepath: the directory where to store the working tree
    :param exec_path: path of the executable
    :param entrypoint: the entrypoint of the analysis
    :param in_paths: the target paths of the analysis
    :param out_paths: the paths to be excluded from the analysis
    :param details: if enable, it returns fine-grained results
    :param tag: label used to mark the data
    :param archive: archive to store/retrieve items

    :raises RepositoryError: raised when there was an error cloning or
        updating the repository.
    """
    version = '0.4.0'

    CATEGORIES = [CATEGORY_COQUA_PYLINT, CATEGORY_COQUA_FLAKE8, CATEGORY_COQUA_JADOLINT]

    def __init__(self, uri, git_path, worktreepath=DEFAULT_WORKTREE_PATH, exec_path=None,
                 entrypoint=None, in_paths=None, out_paths=None, details=False,
                 tag=None, archive=None):
        super().__init__(uri, git_path, worktreepath, exec_path=exec_path,
                         entrypoint=entrypoint, in_paths=in_paths, out_paths=out_paths, details=details,
                         tag=tag, archive=archive)

        self.analyzer_kind = None
        self.analyzer = None

    def fetch(self, category=CATEGORY_COQUA_PYLINT, paths=None,
              from_date=DEFAULT_DATETIME, to_date=DEFAULT_LAST_DATETIME,
              branches=None, latest_items=False):
        """Fetch commits and add code quality information."""

        if not self.entrypoint and category in [CATEGORY_COQUA_FLAKE8, CATEGORY_COQUA_PYLINT]:
            raise GraalError(cause="Entrypoint cannot be null")

        if not self.exec_path and category == CATEGORY_COQUA_JADOLINT:
            raise GraalError(cause="Exec path cannot be null")

        if category == CATEGORY_COQUA_PYLINT:
            self.analyzer_kind = PYLINT
            self.analyzer = ModuleAnalyzer(self.details, self.analyzer_kind)
        elif category == CATEGORY_COQUA_FLAKE8:
            self.analyzer_kind = FLAKE8
            self.analyzer = ModuleAnalyzer(self.details, self.analyzer_kind)
        elif category == CATEGORY_COQUA_JADOLINT:
            self.analyzer_kind = JADOLINT
            self.analyzer = JadolintAnalyzer(self.exec_path, analysis=SMELLS)
        else:
            raise GraalError(cause="Unknown category %s" % category)

        items = super().fetch(category,
                              from_date=from_date, to_date=to_date,
                              branches=branches, latest_items=latest_items)

        return items

    @staticmethod
    def metadata_category(item):
        """Extracts the category from a Code item.

        This backend generates two types of item which can be:
        'code_quality_pylint', 'code_quality_flake8' or 'code_quality_jadolint'
        """
        if item['analyzer'] == PYLINT:
            return CATEGORY_COQUA_PYLINT
        elif item['analyzer'] == FLAKE8:
            return CATEGORY_COQUA_FLAKE8
        elif item['analyzer'] == JADOLINT:
            return CATEGORY_COQUA_JADOLINT
        else:
            raise GraalError(cause="Unknown analyzer %s" % item['analyzer'])

    def _filter_commit(self, commit):
        """Filter a commit according to its data (e.g., author, sha, etc.)

        :param commit: a Perceval commit item

        :returns: a boolean value
        """
        if not self.in_paths:
            return False

        for f in commit['files']:
            for p in self.in_paths:
                if f['file'].endswith(p):
                    return False

        return True

    def _analyze(self, commit):
        """Analyse a snapshot and the corresponding
        checkout version of the repository

        :param commit: a Perceval commit item
        """
        analysis = {}
        if self.analyzer_kind in [FLAKE8, PYLINT]:
            module_path = os.path.join(self.worktreepath, self.entrypoint)

            if not GraalRepository.exists(module_path):
                logger.warning("module path %s does not exist at commit %s, analysis will be skipped"
                               % (module_path, commit['commit']))
                return {}

            analysis = self.analyzer.analyze(module_path, self.worktreepath)
        else:
            for committed_file in commit['files']:
                file_path = committed_file['file']
                if self.in_paths:
                    found = [p for p in self.in_paths if file_path.endswith(p)]
                    if not found:
                        continue

                local_path = self.worktreepath + '/' + file_path
                if not GraalRepository.exists(local_path):
                    analysis.update({file_path: {SMELLS: []}})
                    continue

                smells = self.analyzer.analyze(local_path)
                digested_smells = {SMELLS: [smell.replace(self.worktreepath, '') for smell in smells[SMELLS]]}
                analysis.update({file_path: digested_smells})

        return analysis

    def _post(self, commit):
        """Remove attributes of the Graal item obtained

        :param commit: a Graal commit item
        """
        commit.pop('Author', None)
        commit.pop('Commit', None)
        commit.pop('files', None)
        commit.pop('parents', None)
        commit.pop('refs', None)
        commit['analyzer'] = self.analyzer_kind
        return commit


class JadolintAnalyzer(Analyzer):
    """Class to obtain a list of smells extracted from Dockerfiles."""

    def __init__(self, exec_path, analysis=SMELLS):
        self.analyzer = Jadolint(exec_path, analysis=analysis)

    def analyze(self, file_path):
        """Analyze the content of a Python project using Jadolint

        :param file_path: path of the target file

        :returns a dict containing the results of the analysis, like the one below
        {
          'image_path': ..
        }
        """
        kwargs = {'file_path': file_path}
        analysis = self.analyzer.analyze(**kwargs)

        return analysis


class ModuleAnalyzer(Analyzer):
    """Class to evaluate code quality in a Python project

    :params details: if enable, it returns fine-grained results
    :param kind: the analyzer kind (e.g., PYLINT, FLAKE8)
    """

    def __init__(self, details=False, kind=PYLINT):
        self.details = details
        self.kind = kind

        if kind == PYLINT:
            self.analyzer = PyLint()
        else:
            self.analyzer = Flake8()

    def analyze(self, module_path, worktree_path):
        """Analyze the content of a module

        :param module_path: module path
        :param worktree_path: worktree path

        :returns a dict containing the results of the analysis, like the one below
        {
          'warnings': [..]
        }
        """
        kwargs = {
            'module_path': module_path,
            'details': self.details
        }
        if self.kind == FLAKE8:
            kwargs['worktree_path'] = worktree_path
        analysis = self.analyzer.analyze(**kwargs)

        return analysis


class CoQuaCommand(GraalCommand):
    """Class to run CoQua backend from the command line."""

    BACKEND = CoQua

    @classmethod
    def setup_cmd_parser(cls):
        """Returns the CoQua argument parser."""

        parser = GraalCommand.setup_cmd_parser(cls.BACKEND)

        return parser



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2020 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Valerio Cosentino <valcos@bitergia.com>
#     inishchith <inishchith@gmail.com>
#

import os
import subprocess
import unittest.mock

from base_analyzer import (TestCaseAnalyzer,
                           ANALYZER_TEST_FILE)

from graal.backends.core.analyzers.bandit import Bandit
from graal.graal import GraalError


class TestBandit(TestCaseAnalyzer):
    """Bandit tests"""

    def test_analyze_details(self):
        """Test whether bandit returns the expected fields data"""

        bandit = Bandit()
        kwargs = {
            'folder_path': os.path.join(self.repo_path),
            'details': True
        }
        result = bandit.analyze(**kwargs)

        self.assertIn('loc_analyzed', result)
        self.assertTrue(type(result['loc_analyzed']), int)
        self.assertIn('num_vulns', result)
        self.assertTrue(type(result['num_vulns']), int)
        self.assertIn('by_severity', result)
        self.assertTrue(type(result['by_severity']), dict)
        self.assertIn('undefined', result['by_severity'])
        self.assertTrue(type(result['by_severity']['undefined']), int)
        self.assertIn('low', result['by_severity'])
        self.assertTrue(type(result['by_severity']['low']), int)
        self.assertIn('medium', result['by_severity'])
        self.assertTrue(type(result['by_severity']['medium']), int)
        self.assertIn('high', result['by_severity'])
        self.assertTrue(type(result['by_severity']['high']), int)

        self.assertIn('by_confidence', result)
        self.assertTrue(type(result['by_confidence']), dict)
        self.assertIn('undefined', result['by_confidence'])
        self.assertTrue(type(result['by_confidence']['undefined']), int)
        self.assertIn('low', result['by_confidence'])
        self.assertTrue(type(result['by_confidence']['low']), int)
        self.assertIn('medium', result['by_confidence'])
        self.assertTrue(type(result['by_confidence']['medium']), int)
        self.assertIn('high', result['by_confidence'])
        self.assertTrue(type(result['by_confidence']['high']), int)

        self.assertIn('vulns', result)

        vd = result['vulns'][0]
        self.assertIn('file', vd)
        self.assertTrue(type(vd['file']), str)
        self.assertIn('line', vd)
        self.assertTrue(type(vd['line']), int)
        self.assertIn('severity', vd)
        self.assertTrue(type(vd['severity']), str)
        self.assertIn('confidence', vd)
        self.assertTrue(type(vd['confidence']), str)
        self.assertIn('descr', vd)
        self.assertTrue(type(vd['descr']), str)

    def test_analyze_no_details(self):
        """Test whether bandit returns the expected fields data"""

        bandit = Bandit()
        kwargs = {
            'folder_path': os.path.join(self.repo_path, ANALYZER_TEST_FILE),
            'details': False
        }
        result = bandit.analyze(**kwargs)

        self.assertIn('loc_analyzed', result)
        self.assertTrue(type(result['loc_analyzed']), int)
        self.assertIn('num_vulns', result)
        self.assertTrue(type(result['num_vulns']), int)
        self.assertIn('by_severity', result)
        self.assertTrue(type(result['by_severity']), dict)
        self.assertIn('undefined', result['by_severity'])
        self.assertTrue(type(result['by_severity']['undefined']), int)
        self.assertIn('low', result['by_severity'])
        self.assertTrue(type(result['by_severity']['low']), int)
        self.assertIn('medium', result['by_severity'])
        self.assertTrue(type(result['by_severity']['medium']), int)
        self.assertIn('high', result['by_severity'])
        self.assertTrue(type(result['by_severity']['high']), int)

        self.assertIn('by_confidence', result)
        self.assertTrue(type(result['by_confidence']), dict)
        self.assertIn('undefined', result['by_confidence'])
        self.assertTrue(type(result['by_confidence']['undefined']), int)
        self.assertIn('low', result['by_confidence'])
        self.assertTrue(type(result['by_confidence']['low']), int)
        self.assertIn('medium', result['by_confidence'])
        self.assertTrue(type(result['by_confidence']['medium']), int)
        self.assertIn('high', result['by_confidence'])
        self.assertTrue(type(result['by_confidence']['high']), int)

        self.assertNotIn('vulns', result)

    @unittest.mock.patch('subprocess.check_output')
    def test_analyze_error(self, check_output_mock):
        """Test whether an exception is thrown in case of errors"""

        check_output_mock.side_effect = subprocess.CalledProcessError(-1, "command", output=b'output')

        bandit = Bandit()
        kwargs = {
            'folder_path': os.path.join(self.repo_path, ANALYZER_TEST_FILE),
            'details': False
        }
        with self.assertRaises(GraalError):
            _ = bandit.analyze(**kwargs)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2020 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Valerio Cosentino <valcos@bitergia.com>
#     inishchith <inishchith@gmail.com>
#

import os
import shutil
import subprocess
import tempfile
import unittest

ANALYZER_TEST_FOLDER = "data/"
ANALYZER_TEST_FILE = "sample_code.py"
DOCKERFILE_TEST = "Dockerfile"


def get_file_path(filename):
    return os.path.join(os.path.dirname(os.path.abspath(__file__)), filename)


class TestCaseAnalyzer(unittest.TestCase):
    """Base class to test Analyzers"""

    @classmethod
    def setUpClass(cls):
        cls.tmp_path = tempfile.mkdtemp(prefix='graal_')

        data_path = os.path.dirname(os.path.abspath(__file__))
        cls.tmp_data_path = os.path.join(data_path, 'data')

        repo_name = 'graaltest'
        cls.repo_path = os.path.join(cls.tmp_path, repo_name)

        fdout, _ = tempfile.mkstemp(dir=cls.tmp_path)

        zip_path = os.path.join(cls.tmp_data_path, repo_name + '.zip')
        subprocess.check_call(['unzip', '-qq', zip_path, '-d', cls.tmp_path])

        cls.origin_path = os.path.join(cls.tmp_path, repo_name)

        # copy file to tmp_path
        test_analyzer = get_file_path(ANALYZER_TEST_FOLDER + ANALYZER_TEST_FILE)
        shutil.copy2(test_analyzer, cls.tmp_path)

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.tmp_path)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2020 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Valerio Cosentino <valcos@bitergia.com>
#     inishchith <inishchith@gmail.com>
#

import os
import shutil
import subprocess
import tempfile
import unittest.mock

from graal.graal import GraalCommandArgumentParser
from graal.backends.core.analyzers.jadolint import Jadolint, DEPENDENCIES
from graal.backends.core.analyzers.reverse import Reverse
from graal.backends.core.codep import (CATEGORY_CODEP_PYREVERSE,
                                       CATEGORY_CODEP_JADOLINT,
                                       CoDep,
                                       PyreverseAnalyzer,
                                       JadolintAnalyzer,
                                       CoDepCommand,
                                       logger)
from graal.graal import GraalError
from perceval.utils import DEFAULT_DATETIME
from base_analyzer import (TestCaseAnalyzer,
                           ANALYZER_TEST_FOLDER,
                           DOCKERFILE_TEST,
                           get_file_path)
from base_repo import TestCaseRepo
from utils import JADOLINT_PATH


class TestCoDepBackend(TestCaseRepo):
    """CoDep backend tests"""

    def test_fetch_unknown_category(self):
        """Test whether an exception is thrown when the category is unknown"""

        cd = CoDep('http://example.com', self.git_path, self.worktree_path, tag='test')
        with self.assertRaises(GraalError):
            _ = [item for item in cd.fetch(category="unknown")]

    def test_metadata_category(self):
        """Test metadata_category"""
        item = {
            "Author": "Nishchith Shetty <inishchith@gmail.com>",
            "AuthorDate": "Tue Feb 26 22:06:31 2019 +0530",
            "Commit": "Nishchith Shetty <inishchith@gmail.com>",
            "CommitDate": "Tue Feb 26 22:06:31 2019 +0530",
            "analysis": [],
            "analyzer": "pyreverse",
            "commit": "5866a479587e8b548b0cb2d591f3a3f5dab04443",
            "message": "[copyright] Update copyright dates"
        }
        self.assertEqual(CoDep.metadata_category(item), CATEGORY_CODEP_PYREVERSE)

        item = {
            "Author": "Nishchith Shetty <inishchith@gmail.com>",
            "AuthorDate": "Tue Feb 26 22:06:31 2019 +0530",
            "Commit": "Nishchith Shetty <inishchith@gmail.com>",
            "CommitDate": "Tue Feb 26 22:06:31 2019 +0530",
            "analysis": [],
            "analyzer": "jadolint",
            "commit": "5866a479587e8b548b0cb2d591f3a3f5dab04443",
            "message": "[copyright] Update copyright dates"
        }
        self.assertEqual(CoDep.metadata_category(item), CATEGORY_CODEP_JADOLINT)

        item = {
            "Author": "Nishchith Shetty <inishchith@gmail.com>",
            "AuthorDate": "Tue Feb 26 22:06:31 2019 +0530",
            "Commit": "Nishchith Shetty <inishchith@gmail.com>",
            "CommitDate": "Tue Feb 26 22:06:31 2019 +0530",
            "analysis": [],
            "analyzer": "unknown",
            "commit": "5866a479587e8b548b0cb2d591f3a3f5dab04443",
            "message": "[copyright] Update copyright dates"
        }
        with self.assertRaises(GraalError):
            _ = CoDep.metadata_category(item)


class TestCoDepPyReverseBackend(TestCaseRepo):
    """CoDep backend tests"""

    def test_initialization(self):
        """Test whether attributes are initializated"""

        cd = CoDep('http://example.com', self.git_path, self.worktree_path, entrypoint="module", tag='test')
        self.assertEqual(cd.uri, 'http://example.com')
        self.assertEqual(cd.gitpath, self.git_path)
        self.assertEqual(cd.worktreepath, os.path.join(self.worktree_path, os.path.split(cd.gitpath)[1]))
        self.assertEqual(cd.origin, 'http://example.com')
        self.assertEqual(cd.tag, 'test')
        self.assertEqual(cd.entrypoint, "module")

    def test_fetch(self):
        """Test whether commits are properly processed"""

        cd = CoDep('http://example.com', self.git_path, self.worktree_path, entrypoint="perceval")
        commits = [commit for commit in cd.fetch()]

        self.assertEqual(len(commits), 6)
        self.assertFalse(os.path.exists(cd.worktreepath))

        commit = commits[0]
        self.assertEqual(commit['backend_name'], 'CoDep')
        self.assertEqual(commit['category'], CATEGORY_CODEP_PYREVERSE)
        result = commit['data']['analysis']
        self.assertIn('classes', result)
        self.assertTrue(type(result['classes']), dict)
        self.assertIn('nodes', result['classes'])
        self.assertTrue(type(result['classes']['nodes']), list)
        self.assertIn('links', result['classes'])
        self.assertTrue(type(result['classes']['links']), list)
        self.assertIn('packages', result)
        self.assertTrue(type(result['packages']), dict)
        self.assertTrue(type(result['packages']['nodes']), list)
        self.assertIn('links', result['packages'])
        self.assertTrue(type(result['packages']['links']), list)

    def test_fetch_not_existing_module(self):
        """Test whether warning messages are logged when a module is not found"""

        cd = CoDep('http://example.com', self.git_path, self.worktree_path, entrypoint="unknown")

        with self.assertLogs(logger, level='WARNING') as cm:
            for commit in cd.fetch():
                self.assertRegex(cm.output[-1], 'module path .* does not exist .* analysis will be skipped')
                self.assertEqual(commit['data']['analysis'], {})

    def test_fetch_error(self):
        """Test whether an exception is thrown when the entrypoint isn't defined and the category is pyreverse"""

        cd = CoDep('http://example.com', self.git_path, self.worktree_path, details=True, tag='test')
        with self.assertRaises(GraalError):
            _ = [item for item in cd.fetch()]


class TestCoDepJadolintBackend(TestCaseRepo):
    """CoDep backend tests"""

    repo_name = 'graaltest-dockerfile'

    def setUp(self):
        super().setUp()

    def test_initialization(self):
        """Test whether attributes are initializated"""

        cd = CoDep('http://example.com', self.git_path, self.worktree_path, exec_path=JADOLINT_PATH, tag='test')
        self.assertEqual(cd.uri, 'http://example.com')
        self.assertEqual(cd.gitpath, self.git_path)
        self.assertEqual(cd.worktreepath, os.path.join(self.worktree_path, os.path.split(cd.gitpath)[1]))
        self.assertEqual(cd.origin, 'http://example.com')
        self.assertEqual(cd.tag, 'test')
        self.assertEqual(cd.exec_path, JADOLINT_PATH)

    def test_fetch(self):
        """Test whether commits are properly processed"""

        cd = CoDep('http://example.com', self.git_path, self.worktree_path,
                   exec_path=JADOLINT_PATH, in_paths=['Dockerfile'])
        commits = [commit for commit in cd.fetch(category=CATEGORY_CODEP_JADOLINT)]

        self.assertEqual(len(commits), 2)

        analysis = commits[0]['data']['analysis']
        self.assertIn('Dockerfile', analysis)
        expected_deps = ['debian stretch-slim',
                         'bash',
                         'locales',
                         'gcc',
                         'git',
                         'git-core',
                         'python3',
                         'python3-pip',
                         'python3-venv',
                         'python3-dev',
                         'python3-gdbm',
                         'mariadb-client',
                         'unzip',
                         'curl',
                         'wget',
                         'sudo',
                         'ssh']
        analysis['Dockerfile'][DEPENDENCIES].sort()
        expected_deps.sort()
        self.assertListEqual(analysis['Dockerfile'][DEPENDENCIES], expected_deps)

        analysis = commits[1]['data']['analysis']
        self.assertIn('Dockerfile', analysis)
        expected_deps = ['debian stretch-slim',
                         'bash',
                         'locales',
                         'gcc',
                         'git',
                         'cloc',
                         'git-core',
                         'python3',
                         'python3-pip',
                         'python3-venv',
                         'python3-dev',
                         'python3-gdbm',
                         'mariadb-client',
                         'unzip',
                         'curl',
                         'wget',
                         'sudo',
                         'ssh']
        analysis['Dockerfile'][DEPENDENCIES].sort()
        expected_deps.sort()
        self.assertListEqual(analysis['Dockerfile'][DEPENDENCIES], expected_deps)

    def test_fetch_empty(self):
        """Test whether no commits are returned"""

        cd = CoDep('http://example.com', self.git_path, self.worktree_path,
                   exec_path=JADOLINT_PATH, in_paths=['Unknown'])
        commits = [commit for commit in cd.fetch(category=CATEGORY_CODEP_JADOLINT)]
        self.assertEqual(commits, [])

    def test_fetch_error(self):
        """Test whether an exception is thrown when the exec_path isn't defined and the category is jadolint"""

        cd = CoDep('http://example.com', self.git_path, self.worktree_path, tag='test')
        with self.assertRaises(GraalError):
            _ = [item for item in cd.fetch(category=CATEGORY_CODEP_JADOLINT)]


class TestDependencyAnalyzer(TestCaseAnalyzer):
    """DependencyAnalyzer tests"""

    @classmethod
    def setUpClass(cls):
        cls.tmp_path = tempfile.mkdtemp(prefix='codep_')

        data_path = os.path.dirname(os.path.abspath(__file__))
        data_path = os.path.join(data_path, 'data')

        repo_name = 'graaltest'
        cls.repo_path = os.path.join(cls.tmp_path, repo_name)

        fdout, _ = tempfile.mkstemp(dir=cls.tmp_path)

        zip_path = os.path.join(data_path, repo_name + '.zip')
        subprocess.check_call(['unzip', '-qq', zip_path, '-d', cls.tmp_path])

        test_analyzer = get_file_path(ANALYZER_TEST_FOLDER + DOCKERFILE_TEST)
        shutil.copy2(test_analyzer, cls.tmp_path)

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.tmp_path)

    def test_pyreverse_init(self):
        """Test initialization"""

        dep_analyzer = PyreverseAnalyzer()

        self.assertIsInstance(dep_analyzer, PyreverseAnalyzer)
        self.assertIsInstance(dep_analyzer.analyzer, Reverse)

    def test_jadolint_init(self):
        """Test initialization"""

        dep_analyzer = JadolintAnalyzer(JADOLINT_PATH)

        self.assertIsInstance(dep_analyzer, JadolintAnalyzer)
        self.assertIsInstance(dep_analyzer.analyzer, Jadolint)

    def test_analyze_pyreverse(self):
        """Test whether the analyze method works"""

        module_path = os.path.join(self.tmp_path, 'graaltest', 'perceval')
        dep_analyzer = PyreverseAnalyzer()
        result = dep_analyzer.analyze(module_path)

        self.assertIn('classes', result)
        self.assertTrue(type(result['classes']), dict)
        self.assertIn('nodes', result['classes'])
        self.assertTrue(type(result['classes']['nodes']), list)
        self.assertIn('links', result['classes'])
        self.assertTrue(type(result['classes']['links']), list)

    def test_analyze_jadolint(self):
        """Test whether the analyze method works"""

        file_path = os.path.join(self.tmp_path, DOCKERFILE_TEST)
        dep_analyzer = JadolintAnalyzer(JADOLINT_PATH)
        result = dep_analyzer.analyze(file_path)

        expected_deps = [
            'debian stretch-slim',
            'bash',
            'locales',
            'gcc',
            'git',
            'git-core',
            'python3',
            'python3-pip',
            'python3-venv',
            'python3-dev',
            'python3-gdbm',
            'mariadb-client',
            'unzip',
            'curl',
            'wget',
            'sudo',
            'ssh'
        ]

        self.assertIn(DEPENDENCIES, result)
        self.assertListEqual(result[DEPENDENCIES], expected_deps)


class TestCoDepCommand(unittest.TestCase):
    """CoDepCommand tests"""

    def test_backend_class(self):
        """Test if the backend class is CoDep"""

        self.assertIs(CoDepCommand.BACKEND, CoDep)

    def test_setup_cmd_parser(self):
        """Test setup_cmd_parser"""

        parser = CoDepCommand.setup_cmd_parser()
        self.assertIsInstance(parser, GraalCommandArgumentParser)
        self.assertEqual(parser._backend, CoDep)

        args = ['http://example.com/',
                '--git-path', '/tmp/gitpath',
                '--tag', 'test',
                '--from-date', '1970-01-01']

        parsed_args = parser.parse(*args)
        self.assertEqual(parsed_args.uri, 'http://example.com/')
        self.assertEqual(parsed_args.git_path, '/tmp/gitpath')
        self.assertEqual(parsed_args.tag, 'test')
        self.assertEqual(parsed_args.from_date, DEFAULT_DATETIME)

        args = ['http://example.com/',
                '--git-path', '/tmp/gitpath',
                '--tag', 'test',
                '--exec-path', JADOLINT_PATH,
                '--category', CATEGORY_CODEP_JADOLINT,
                '--in-paths', 'Dockerfile', 'Dockerfile-full', 'Dockerfile-secured']

        parsed_args = parser.parse(*args)
        self.assertEqual(parsed_args.uri, 'http://example.com/')
        self.assertEqual(parsed_args.git_path, '/tmp/gitpath')
        self.assertEqual(parsed_args.tag, 'test')
        self.assertEqual(parsed_args.from_date, DEFAULT_DATETIME)
        self.assertEqual(parsed_args.category, CATEGORY_CODEP_JADOLINT),
        self.assertEqual(parsed_args.exec_path, JADOLINT_PATH)
        self.assertListEqual(parsed_args.in_paths, ['Dockerfile', 'Dockerfile-full', 'Dockerfile-secured'])


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2020 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     inishchith <inishchith@gmail.com>
#     Valerio Cosentino <valcos@bitergia.com>
#

import subprocess

from .analyzer import Analyzer


class Flake8(Analyzer):
    """A wrapper for Flake8, a source code style checker for Python."""

    version = '0.1.0'

    def analyze(self, **kwargs):
        """Add quality checks data using Flake8.

        :param module_path: module path
        :param details: if True, it returns information about single commit

        :returns result: dict of the results of the analysis
        """
        module_path = kwargs['module_path']
        details = kwargs['details']
        worktree_path = kwargs['worktree_path']

        try:
            msg = subprocess.check_output(
                ['flake8', "--format='%(path)s::%(row)d::%(col)d::%(code)s::%(text)s'", module_path]).decode("utf-8")
        except subprocess.CalledProcessError as e:
            msg = e.output.decode("utf-8")
        finally:
            subprocess._cleanup()

        lines = msg.strip().split('\n') if msg else []

        result = {'warnings': len(lines)}

        if details:
            flake8_verbose = []
            for line in lines:
                path, row, column, type_of_warning, description = line.split("::")
                file_path = path[path.index(worktree_path) + 1:] if path.startswith(worktree_path) else path

                line_details = {
                    "file_path": file_path,
                    "type_of_warning": type_of_warning,
                    "line": row,
                    "column": column,
                    "description": description
                }
                flake8_verbose.append(line_details)

            result['lines'] = flake8_verbose

        return result
